#include <cerrno>
#include <msp/core/systemerror.h>
#include <msp/io/handle_private.h>
#include <msp/strings/format.h>
#include "datagramsocket.h"
#include "sockaddr_private.h"
#include "socket_private.h"

namespace Msp {
namespace Net {

DatagramSocket::DatagramSocket(Family f, int p):
	ClientSocket(f, SOCK_DGRAM, p)
{
#ifdef WIN32
	WSAEventSelect(priv->handle, *priv->event, FD_READ|FD_CLOSE);
#endif
	set_events(IO::P_INPUT);
}

bool DatagramSocket::connect(const SockAddr &addr)
{
	SockAddr::SysAddr sa = addr.to_sys();

	int err = ::connect(priv->handle, reinterpret_cast<sockaddr *>(&sa.addr), sa.size);
	if(err==-1)
	{
#ifdef WIN32
		throw system_error("connect", WSAGetLastError());
#else
		throw system_error("connect");
#endif
	}

	delete peer_addr;
	peer_addr = addr.copy();

	delete local_addr;
	SockAddr::SysAddr lsa;
	getsockname(priv->handle, reinterpret_cast<sockaddr *>(&lsa.addr), &lsa.size);
	local_addr = SockAddr::from_sys(lsa);

	connected = true;

	return true;
}

unsigned DatagramSocket::sendto(const char *buf, unsigned size, const SockAddr &addr)
{
	if(size==0)
		return 0;

	SockAddr::SysAddr sa = addr.to_sys();

	int ret = ::sendto(priv->handle, buf, size, 0, reinterpret_cast<sockaddr *>(&sa.addr), sa.size);
	if(ret<0)
	{
		if(errno==EAGAIN)
			return 0;
		else
		{
#ifdef WIN32
			throw system_error("sendto", WSAGetLastError());
#else
			throw system_error("sendto");
#endif
		}
	}

	return ret;
}

unsigned DatagramSocket::recvfrom(char *buf, unsigned size, SockAddr *&from_addr)
{
	if(size==0)
		return 0;

	SockAddr::SysAddr sa;
	int ret = ::recvfrom(priv->handle, buf, size, 0, reinterpret_cast<sockaddr *>(&sa.addr), &sa.size);
	if(ret<0)
	{
		if(errno==EAGAIN)
			return 0;
		else
		{
#ifdef WIN32
			throw system_error("recvfrom", WSAGetLastError());
#else
			throw system_error("recvfrom");
#endif
		}
	}

	from_addr = SockAddr::from_sys(sa);

	return ret;
}

} // namespace Net
} // namespace Msp
